@@ -1,4 +1,4 @@
-# Copyright (C) 2012-2014, 2016 onpon4 <onpon4@riseup.net>
+# Copyright (C) 2012-2014, 2016 Julie Marchant <onpon4@riseup.net>
 # 
 # This file is part of the Pygame SGE.
 # 
@@ -25,8 +25,11 @@ from __future__ import print_function
 from __future__ import unicode_literals
 
 import os
+import random
+import warnings
 
 import pygame
+import six
 
 import sge
 from sge import r
@@ -61,6 +64,24 @@ class Sound(object):
        will be stopped before playing the new instance.  Set to
        :const:`None` for no limit.
 
+    .. attribute:: parent
+
+       Indicates another sound which is treated as being the same sound
+       as this one for the purpose of determining whether or not, and
+       how many times, the sound is playing.  Set to :const:`None` for
+       no parent.
+
+       If the sound has a parent, :attr:`max_play` will have no effect
+       and instead the parent sound's :attr:`max_play` will apply to
+       both the parent sound and this sound.
+
+       .. warning::
+
+          It is acceptable for a sound to both be a parent and have a
+          parent.  However, there MUST be a parent at the top which has
+          no parent.  The behavior of circular parenting, such as making
+          two sounds parents of each other, is undefined.
+
     .. attribute:: fname
 
        The file name of the sound given when it was created.
@@ -81,19 +102,37 @@ class Sound(object):
 
     @property
     def max_play(self):
-        return len(self.__channels)
+        return len(self.rd["channels"]) if self.rd["channels"] else None
 
     @max_play.setter
     def max_play(self, value):
+        self.__max_play = value
+
         if value is None:
             value = 0
 
-        if self.__sound is not None:
+        if self.__sound is not None and self.parent is None:
             value = max(0, value)
-            while len(self.__channels) < value:
-                self.__channels.append(_get_channel())
-            while len(self.__channels) > value:
-                _release_channel(self.__channels.pop(-1))
+            while len(self.rd["channels"]) < value:
+                self.rd["channels"].append(_get_channel())
+            while len(self.rd["channels"]) > value:
+                _release_channel(self.rd["channels"].pop(-1))
+
+    @property
+    def parent(self):
+        return self.__parent
+
+    @parent.setter
+    def parent(self, value):
+        if value != self.__parent:
+            self.__parent = value
+            if value is None:
+                self.rd["channels"] = []
+                self.max_play = self.__max_play
+            else:
+                while self.rd["channels"]:
+                    _release_channel(self.rd["channels"].pop(-1))
+                self.rd["channels"] = value.rd["channels"]
 
     @property
     def length(self):
@@ -105,20 +144,20 @@ class Sound(object):
     @property
     def playing(self):
         n = 0
-        for channel in self.__channels + self.__temp_channels:
+        for channel in self.rd["channels"] + self.__temp_channels:
             if channel.get_busy():
                 n += 1
 
         return n
 
-    def __init__(self, fname, volume=1, max_play=1):
+    def __init__(self, fname, volume=1, max_play=1, parent=None):
         """
         Arguments:
 
         - ``fname`` -- The path to the sound file.  If set to
           :const:`None`, this object will not actually play any sound.
           If this is neither a valid sound file nor :const:`None`,
-          :exc:`IOError` is raised.
+          :exc:`OSError` is raised.
 
         All other arguments set the respective initial attributes of the
         sound.  See the documentation for :class:`sge.snd.Sound` for
@@ -131,19 +170,24 @@ class Sound(object):
             try:
                 self.__sound = pygame.mixer.Sound(fname)
             except pygame.error as e:
-                raise IOError(e)
+                raise OSError(e)
 
         else:
             self.__sound = None
 
-        self.__channels = []
+        self.rd["channels"] = []
         self.__temp_channels = []
         self.fname = fname
         self.volume = volume
-        self.max_play = max_play
+        self.__parent = None
+        if parent is None:
+            self.max_play = max_play
+        else:
+            self.__max_play = max_play
+            self.parent = parent
 
     def play(self, loops=1, volume=1, balance=0, maxtime=None,
-             fade_time=None):
+             fade_time=None, force=True):
         """
         Play the sound.
 
@@ -163,6 +207,11 @@ class Sound(object):
         - ``fade_time`` -- The time in milliseconds over which to fade
           the sound in; set to :const:`None` or ``0`` to immediately
           play the sound at full volume.
+        - ``force`` -- Whether or not the sound should be played even if
+          it is already playing the maximum number of times.  If set to
+          :const:`True` and the sound is already playing the maximum
+          number of times, one of the instances of the sound already
+          playing will be stopped.
         """
         if self.__sound is not None:
             if not loops:
@@ -184,15 +233,16 @@ class Sound(object):
                 left_volume *= 1 - abs(balance)
 
             if self.max_play:
-                for channel in self.__channels:
+                for channel in self.rd["channels"]:
                     if not channel.get_busy():
                         channel.play(self.__sound, loops, maxtime, fade_time)
                         channel.set_volume(left_volume, right_volume)
                         break
                 else:
-                    self.__channels[0].play(self.__sound, loops, maxtime,
-                                            fade_time)
-                    self.__channels[0].set_volume(left_volume, right_volume)
+                    if force:
+                        channel = random.choice(self.rd["channels"])
+                        channel.play(self.__sound, loops, maxtime, fade_time)
+                        channel.set_volume(left_volume, right_volume)
             else:
                 channel = _get_channel()
                 channel.play(self.__sound, loops, maxtime, fade_time)
@@ -219,12 +269,12 @@ class Sound(object):
 
     def pause(self):
         """Pause playback of the sound."""
-        for channel in self.__channels:
+        for channel in self.rd["channels"]:
             channel.pause()
 
     def unpause(self):
         """Resume playback of the sound if paused."""
-        for channel in self.__channels:
+        for channel in self.rd["channels"]:
             channel.unpause()
 
 
@@ -316,7 +366,7 @@ class Music(object):
         - ``fname`` -- The path to the sound file.  If set to
           :const:`None`, this object will not actually play any music.
           If this is neither a valid sound file nor :const:`None`,
-          :exc:`IOError` is raised.
+          :exc:`OSError` is raised.
 
         All other arguments set the respective initial attributes of the
         music.  See the documentation for :class:`sge.snd.Music` for
@@ -326,7 +376,10 @@ class Music(object):
         if fname is None or os.path.isfile(fname):
             self.fname = fname
         else:
-            raise IOError('File "{}" not found.'.format(fname))
+            if six.PY2:
+                raise OSError('File "{}" not found.'.format(fname))
+            else:
+                raise FileNotFoundError('File "{}" not found.'.format(fname))
         self.volume = volume
         self.rd["timeout"] = None
         self.rd["fade_time"] = None
@@ -347,7 +400,10 @@ class Music(object):
         """
         if self.fname is not None:
             if not self.playing:
-                pygame.mixer.music.load(self.fname)
+                try:
+                    pygame.mixer.music.load(self.fname)
+                except pygame.error as e:
+                    warnings.warn(e)
 
             if not loops:
                 loops = -1
